/*
 * Portions of this file Copyright 1999-2005 University of Chicago 
 * Portions of this file Copyright 1999-2005 The University of Southern California.
 *
 * This file or a portion of this file is licensed under the 
 * terms of the Globus Toolkit Public License, found at
 * http://www.globus.org/toolkit/download/license.html.
 * If you redistribute this file, with or without 
 * modifications, you must include this notice in the file.
 */
package org.globus.mds.aggregator.impl;

import java.util.Date;
import java.util.List;

import javax.naming.Context;
import javax.naming.InitialContext;

import commonj.timers.Timer;
import commonj.timers.TimerListener;
import commonj.timers.TimerManager;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import org.globus.usage.packets.IPTimeMonitorPacket;
import org.globus.usage.packets.CustomByteBuffer;

import org.globus.wsrf.Constants;
import org.globus.wsrf.container.UsageConfig;

public class MDSAggregatorMonitorPacket 
       extends IPTimeMonitorPacket 
       implements TimerListener 
{
    static Log log = LogFactory.getLog(MDSAggregatorMonitorPacket.class);

    private List usageTargets = null;
    private long lifetimeRegistrationCount;
    private long currentRegistrantCount;
    private Date resourceCreationTime;
    private String serviceName;
    
    private Timer timer = null;
    private TimerManager timerMgr = null;
    private boolean started = false;
    
    private static short COMPONENT_CODE = 6;  
    private static short PACKET_VERSION = 0;
    private static short MAX_SERVICE_NAME_LEN = 40;
    
    public MDSAggregatorMonitorPacket() 
    {
        setComponentCode(COMPONENT_CODE);
        setPacketVersion(PACKET_VERSION);
        try {
            UsageConfig usageConfig = UsageConfig.getUsageConfig();
            if (usageConfig != null) {
                this.usageTargets = usageConfig.getTargets();
                this.setHostIP(usageConfig.getContainerHost()); 
            }
        } catch (Exception e) {
            log.debug("Exception while getting usage stats config, usage stats will be disabled: " + e);
        }
        
        try { 
            Context initialContext = new InitialContext();
            this.timerMgr = (TimerManager)
                initialContext.lookup(Constants.DEFAULT_TIMER);        
        } catch (Exception e) {
            log.debug("Exception getting TimerManager, usage stats will be disabled: " + e);
            // setting this.usageTargets back to null will keep timers from being started
            this.usageTargets = null;  
        }
    }

    public long getLifetimeRegistrationCount() 
    {
        return this.lifetimeRegistrationCount;
    }
    
    public void setLifetimeRegistrationCount(long lifetimeRegistrationCount) 
    {
        this.lifetimeRegistrationCount = lifetimeRegistrationCount;
    }
    
    public long getCurrentRegistrantCount() 
    {
        return this.currentRegistrantCount;
    }
    
    public void setCurrentRegistrantCount(long currentRegistrantCount) 
    {
        this.currentRegistrantCount = currentRegistrantCount;
    }
    
    public Date getResourceCreationTime() 
    {
        return this.resourceCreationTime;
    }
    
    public void setResourceCreationTime(Date resourceCreationTime) 
    {
        this.resourceCreationTime = resourceCreationTime;
    }

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }
    
    public String getServiceName() {
        return this.serviceName;
    }
    
    public void packCustomFields(CustomByteBuffer buf) 
    {
        super.packCustomFields(buf);
        
        byte[] serviceNameBytes = this.serviceName.getBytes();
        byte[] fixedServiceNameBytes = new byte[MAX_SERVICE_NAME_LEN];        
        for (int i=0; i< MAX_SERVICE_NAME_LEN; i++) {
            if (serviceNameBytes.length > i)
                fixedServiceNameBytes[i] = serviceNameBytes[i];
        }
        buf.put(fixedServiceNameBytes);
        
        buf.putLong(this.lifetimeRegistrationCount);
        buf.putLong(this.currentRegistrantCount);
        buf.putLong(this.resourceCreationTime.getTime());
    }

    public void unpackCustomFields(CustomByteBuffer buf) 
    {
        super.unpackCustomFields(buf);
        
        byte[] fixedServiceNameBytes = new byte[MAX_SERVICE_NAME_LEN];
	buf.get(fixedServiceNameBytes);
	this.serviceName = new String(fixedServiceNameBytes);
        
        this.lifetimeRegistrationCount = buf.getLong();
        this.currentRegistrantCount = buf.getLong();
        this.resourceCreationTime = new Date(buf.getLong());
    }
    
    private void sendUsagePacket() 
    {
        try {
            if (this.usageTargets != null) {
                this.setTimestamp(System.currentTimeMillis());
                this.sendPacket(this.usageTargets);
                if (log.isDebugEnabled()) {
                    log.debug("Sent Usage Packet");
                    this.debug();
                }                   
            }         
        } catch (Exception e) {
            log.warn("Could not send usage packet "+ e);
        }
    }
    
    public void startTimer() 
    {        
        if ((!this.started) && (this.usageTargets != null)) {
            this.timer = 
                this.timerMgr.schedule(this,60000,3600000);  // every hour ?
            this.started = true;
        }
    }
    
    public void stopTimer() 
    {
        if (this.started) {
            this.timer.cancel();
            this.started = false;
        }
    }
    
    public void timerExpired(Timer timer) 
    {
        this.sendUsagePacket();
    }    
    
    public void finalize() 
    {
        this.stopTimer();
    }
    
    public String toString() 
    {
        return super.toString() +
        " Service Name: " + this.serviceName + 
        " Total Lifetime Registrations: " + this.lifetimeRegistrationCount +
        " Current Registrant Count: " + this.currentRegistrantCount +
        " Resource Creation Time: " + this.resourceCreationTime;
    }
    
    public void debug() 
    {
        log.debug(this.toString());
    }
    
    public void display() {
        log.info(this.toString());
    }      
} 
